/*

FLARToolkit example in Away3d

Demonstrates:

How to setup the FLARToolkit with the Away3d engine.
How to define trackable objects with FLARBaseNode
How to reset the camera source in actionscript

code by Rob Bateman
rob@infiniteturtles.co.uk
http://www.infiniteturtles.co.uk

This code is distributed under the MIT License

Copyright (c)  

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the “Software”), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

package
{
	import away3d.animators.data.*;
	import away3d.cameras.*;
	import away3d.containers.*;
	import away3d.core.base.*;
	import away3d.core.clip.*;
	import away3d.core.render.*;
	import away3d.core.utils.*;
	import away3d.events.*;
	import away3d.lights.*;
	import away3d.loaders.*;
	import away3d.materials.*;
	
	import flash.display.*;
	import flash.events.*;
	import flash.filters.*;
	import flash.media.*;
	
	import mx.core.UIComponent;
	
	import org.libspark.flartoolkit.away3d.*;
	import org.libspark.flartoolkit.core.*;
	import org.libspark.flartoolkit.core.param.*;
	import org.libspark.flartoolkit.core.raster.rgb.*;
	import org.libspark.flartoolkit.core.transmat.*;
	import org.libspark.flartoolkit.detector.*;
	
	public class AwayFLARView extends UIComponent
	{
		//camera configuration
		[Embed(source="../assets/camera_para.dat", mimeType='application/octet-stream')]
		private var CameraDataClass:Class;
		
		//marker configuration
		[Embed(source="../assets/FLAR-marker.pat", mimeType='application/octet-stream')]
		private var PatternDataClass:Class;
		
		//ogre MD2 animation
		[Embed(source="../assets/ogre.md2", mimeType='application/octet-stream')]
		private var Model:Class;
		
		//ogre texture
		[Embed(source="../assets/ogre.jpg")]
		private var Texture:Class;
		
		//scene objects
    	private var scene:Scene3D;
		private var camera:FLARCamera3D;
		private var view:View3D;
		
		//config properties
		private var screenWidth:Number = 400;
		private var screenHeight:Number = 300;
		private var threshold:Number = 80;
		private var markerWidth:Number = 80;
		
		//FLARToolkit object
		private var fBaseNode:FLARBaseNode;
		private var fParam:FLARParam;
		private var fCode:FLARCode;
		private var fWebcam:Camera;
		private var fVideo:Video;
		private var fCapture:Bitmap;
		private var fDetector:FLARSingleMarkerDetector;
		private var fResult:FLARTransMatResult;
		private var fRaster:FLARRgbRaster_BitmapData;
		
		//marterial objects
		private var material:BitmapMaterial;
		
		//scene objects
		private var model:Mesh;
		
		//enterframe variables
		private var blurFilter:BlurFilter = new BlurFilter(10, 10, BitmapFilterQuality.LOW);
		public var update:Boolean = false;
		
		public function set webcam(val:String):void
		{
			//set camera based on name string
			if (val) {
				var n:int = Camera.names.length;
				for (var i:int = 0; i < n; i++)
					if (Camera.names[i] == val)
						fWebcam = Camera.getCamera(String(i));
			} else {
				fWebcam = Camera.getCamera();
			}
			
			//clear current video contents
			fVideo.attachCamera(null);
			fVideo.clear();
			
			if (!fWebcam)
				return;
			
			//setup webcam
			fWebcam.setMode(screenWidth, screenHeight, 30);
			fVideo.attachCamera(fWebcam);
		}
		
		public function get webcam():String
		{
			return fWebcam.name;
		}
		
		/**
		 * Constructor
		 */
		public function AwayFLARView()
		{
			init();
		}
		
		/**
		 * Global initialise function
		 */
		private function init():void
		{
			initFLARToolkit();
			initEngine();
			initMaterials();
			initObjects();
		}
		
		/**
		 * Initialise the engine
		 */
		private function initEngine():void
		{
			scene = new Scene3D();
			
			view = new View3D({scene:scene, camera:camera});
			view.x = screenWidth - 5;
			view.y = screenHeight;
			view.addSourceURL("srcview/index.html");
			addChild(view);
		}
		
		/**
		 * Initialise the FLARToolkit
		 */
		private function initFLARToolkit():void
		{
			//setup camera parameters
			fParam = new FLARParam();
			fParam.loadARParam(new CameraDataClass());
			fParam.changeScreenSize(screenWidth, screenHeight);
			camera = new FLARCamera3D();
			camera.setParam(fParam);
			
			//setup marker code
			fCode = new FLARCode(16, 16);
			fCode.loadARPatt(new PatternDataClass());
			
			//setup marker detector
			fDetector = new FLARSingleMarkerDetector(fParam, fCode, markerWidth);
			fDetector.setContinueMode(false);
			fResult = new FLARTransMatResult();
			
			//setup webcam
			fVideo = new Video(screenWidth, screenHeight);
			
			//setup output capture
			fCapture = new Bitmap(new BitmapData(screenWidth, screenHeight, false, 0), PixelSnapping.AUTO, true);
			fCapture.x = 0;
			fCapture.y = 0;
			fCapture.scaleX = 2.0;
			fCapture.scaleY = 2.0;
			addChild(fCapture);
			
			//setup output bitmapData
			fRaster = new FLARRgbRaster_BitmapData(fCapture.bitmapData);
		}
		
		/**
		 * Initialise the materials
		 */
		private function initMaterials():void
		{
			material = new BitmapMaterial(Cast.bitmap(Texture));
		}
		
		/**
		 * Initialise the scene objects
		 */
		private function initObjects():void
		{
			//setup baseNode
			fBaseNode = new FLARBaseNode();
			scene.addChild(fBaseNode);
			
			//setup model
			model = Md2.parse(Model, {material:material}) as Mesh;
			model.scale(0.02);
			model.y += 45;
			model.play(new AnimationSequence("stand", true, true, 10));
			fBaseNode.addChild(model);
		}
		
		/**
		 * Initialise the listeners
		 */
		public function initListeners():void
		{
			addEventListener( Event.ENTER_FRAME, onEnterFrame );
		}
		
		/**
		 * Navigation and render loop
		 */
		private function onEnterFrame(event:Event):void
		{
			fCapture.bitmapData.draw(fVideo);
			
			if (update) {
				try {
					//check for marker
					if (fDetector.detectMarkerLite(fRaster, threshold) && fDetector.getConfidence() > 0.5) {
						view.visible = true;
						fDetector.getTransformMatrix(fResult);
						fBaseNode.setTransformMatrix(fResult);
					} else {
						view.visible = false;
					}
				} catch (e:Error) {
					trace("Error: " + e.message);
				}
				
				view.render();
				
			} else {
				fCapture.bitmapData.applyFilter(fCapture.bitmapData, fCapture.bitmapData.rect, fCapture.bitmapData.rect.topLeft, blurFilter);
			}
		}
	}
}